import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * An enemy actor that flitters around the stage until killed by an explosion or player. A mobile 'mine'.
 * 
 * @Andrew I Letailleur
 * @15/03/2015, Version 2-04
 */
public class Bat extends Actor
{
    //class level attributes
    private int beat, dying;
    
    /**
     * Sets up the Bat's innitial appearance
     */
    public Bat()
    {//local variables
        //Original Image Size is set at 96 x 131
        GreenfootImage image = getImage();
        image.scale(image.getWidth() / 4, image.getHeight() / 4);
        setImage(image);
        
        dying = 2;//sets up the bat's conditional death trigger
    }//end of method bat
    
    /**
     * Act - do whatever the Bat wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        movement();//moves the bat in a random direction
        if (dying < 2)//will count down if dying is below 2
            dying--;
    }//end of method act

    /**
    * will play a sound file and create a delay
    * to allow the file to play
    */
    private void play(String sound, int delayTime)
    {//will play a sound file and create a delay
        //to allow the file to play
        Greenfoot.playSound(sound);
        Greenfoot.delay(delayTime);
    }//end of method Play
    
    /**
     * Rolls a random number to determine the direction the Bat will move in
     */
    public void beatRoll()
    {//local variables
        beat = Greenfoot.getRandomNumber(9);//generates a random number for the beat value
    }//end of method beatRoll

    /**
     * Moves the bat in a random direction
     */
    public void movement()
    {//local variables
        beatRoll();//rolls the beatRoll method to determine the value of 'beat'
        if (beat == 0) {
            setLocation(getX(), getY() -2);
        } else if (beat == 1) {
            setLocation(getX() -2, getY() -2);
        } else if (beat == 2) {
            setLocation(getX() -2, getY());
        } else if (beat == 3) {
            setLocation(getX() -2, getY() +2);
        } else if (beat == 4) {
            setLocation(getX(), getY() +2);
        } else if (beat == 5) {
            setLocation(getX() +2, getY() +2);
        } else if (beat == 6) {
            setLocation(getX() +2, getY());
        } else if (beat == 7) {
            setLocation(getX() +2, getY() -2);
        }//endif movement test
        checkCollision();
    }//end of method movement
    
    /**
     * Checks to see if the bat is touching the edge of the world
     */
    public void checkCollision()
    {
        if (getX() <= 5) {
            setLocation(getX() +2, getY());
        } else if (getX() >= getWorld().getWidth() - 5) {
            setLocation(getX() -2, getY());
        } else if (getY() <= 5) {
            setLocation(getX(), getY() +2);
        } else if (getY() >= getWorld().getWidth() - 5) {
            setLocation(getX(), getY() -2);
        }//endif atWorldEdge
        checkHit();//then checks to see if bat is hit by an enemy object/hazard
    }//end of method checkCollision
    
    /**
     * Checks to see if an explosion is touching an actor/object that can trigger death earlier
     */
    public void checkHit()
    {
        //returns actors that are touching the bat
        Actor shot = getOneIntersectingObject(Bullet.class);
        Actor bang = getOneIntersectingObject(Explosion.class);
        Actor star = getOneIntersectingObject(StarBang.class);
       if (shot != null)//first checks to see if bat is hit by a bullet
            bulletBat();
        else if ( (bang != null) || (star != null) )//As bats can be harmed by explosions
            dying--;//reduces life, triggering a death condition by next act
        else if (dying <=0)
            killBat();//removes the actor fromthis wolrd
    }//end of method checkHit
    
    /**
     * Kills the bat if a bullet object hits the bat, adding score on top of reducing the population total
     */
    public void bulletBat()
    {
        play("batDies.wav", 0);//plays a squick every time
        Gameplay thisWorld = (Gameplay) getWorld();//calls upon the Gameplay world for alterations
        //adds a flash where the bat's location is at, to represent a bullet hitting it's mark
        Flash flash = new Flash();
        thisWorld.addObject(flash, getX(), getY());
        
        thisWorld.removeObject(this);//removes itself from world
        thisWorld.batCap--; //Reduces the current bat and max population, to reflect the dying bat
        thisWorld.maxxCap--;
        thisWorld.killCount++;//adds to kill counter for next level
        thisWorld.Bat();//adds to score for end of game (if implemented)
    }//end of method bulletBat
    
    /**
     * Kills the bat if an explosion hits the bat, reducing the total population after the bat dies
     */
    public void killBat()
    {
        Gameplay thisWorld = (Gameplay) getWorld();//calls upon the Gameplay world for alterations
        thisWorld.removeObject(this);//removes itself from world
        //Reduces the current bat and max population, to reflect the dying bat
        thisWorld.batCap--;
        thisWorld.maxxCap--;
    }//end of method killBat   
}//end of actor Bat