import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * An enemy actor that explodes after time has past, or if a projectile hits it
 * Bombs spawn explosions upon their removal
 * 
 * @Andrew I Letailleur
 * @15/03/2015, Version 2-04
 */
public class Bomb extends Actor
{
    //class level attributes
    private int fuse = 240;
    private int thisX, thisY;

    /**
     * Sets up the Bomb's innitial appearance
     */
    public Bomb()
    {//local variables
        //Original Image Size is set at 96 x 131
        GreenfootImage image = getImage();
        image.scale(image.getWidth() / 4, image.getHeight() / 4);
        setImage(image);
    }//end of method Bomb
    /**
     * Act - do whatever the Bomb wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {//local variables
        countdown();//counts the fuse attribute
    }//end of method act
    
    /**
     * Counts down the timer until it reaches 0, afterwards it will replace the bomb with an explosion
     * Also checks to see if any bullets or explosions trigger another explosion from the bomb
     */
    public void countdown()
    {//local variables
        if (fuse ==0)
            detonate();//causes the bomb to explode
        else {
            fuse--;//reduces the fuse until it reaches 0
            checkCollision();//will then check to see if any touching objects trigger an explosion
        }//endif timer
    }//end of method countdown

    /**
     * Checks to see if the bomb is touching an actor/object that can trigger an explosion early
     */
    private void checkCollision()
    {//local variables
        //returns actors that are touching the Bomb
        Actor shot = getOneIntersectingObject(Bullet.class);
        Actor shit = getOneIntersectingObject(Pellet.class);
        Actor A = getOneIntersectingObject(StarBang.class);
        Actor B = getOneIntersectingObject(Explosion.class);
        if (shot != null)//A tiered check that first checks if bomb is hit by a bullet
            detonate();//causes the bomb to explode
        else if (shit != null)//Then checks to see if bomb is hit by a pellet
            detonate();//causes the bomb to explode
        else if (A != null)//Then checks to see if bomb is hit by a StarBang
            detonate();//causes the bomb to explode
        else if (B != null)//Then finally checks to see if bomb is hit by an Explosion
            detonate();//causes the bomb to explode
        //endif collision priority
    }//end of method checkCollision
    
    /**
     * Removes bomb from game, and replaces it with an explosion
     */
    private void detonate() //hope this fixes possible glitches
    {
        Gameplay thisWorld = (Gameplay) getWorld();//calls upon the Gameplay world for alterations
        //adds an explosion where the bomb's location is at
        Explosion explosion = new Explosion();
        thisWorld.addObject(explosion, getX(), getY());
        thisWorld.removeObject(this);//removes itself from world
    }//end of method detonate
}//end of actor Bomb
