import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Explosion appears after a bomb's timer runs out, or the bomb is hit by a projectile
 * Coded to kill quite a few hazards, and shrink as the explosion 'disappears'
 * 
 * @Andrew I Letailleur
 * @15/03/2015, Version 2-04
 */
public class Explosion extends Blast
{
    //class level attributes
    protected int timer = 24;
    /**
     * Sets up the Explosion's innitial appearance, along with playing a sound on appearing in game
     */
    public Explosion()
    {//local variables
        //Original Image Size is set at 284 x 283
        GreenfootImage image = getImage();
        image.scale(image.getWidth() -120, image.getHeight() -120);
        //consider -120 once gameplay balancing starts, if the explosions are 'too' easy
        setImage(image);
        
        play("Explosion.wav", 0);//plays an explosion sound on set up
    }//end of method Explosion
    
    /**
     * Act - do whatever the Explosion wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {//local variables
        GreenfootImage image = getImage();
        image.scale(image.getWidth() -6, image.getHeight() -6);
        //consider -120 once gameplay balancing starts, if the explosions are 'too' easy
        setImage(image);
        //turn(3);
        countdown();
    }//end of method act
    
    /**
     * Counts down the timer until it reaches 0, afterwards it will remove the explosion and bomb count
     */
    public void countdown()
    {//local variables
        if (timer == 0) {//the explosion will vanish once it's timer is at 0
            Gameplay thisWorld = (Gameplay) getWorld();//calls upon the Gameplay world for alterations
            //reduces the current bomb and max population, to reflect the disappearing explosion
            thisWorld.bombCap--;
            thisWorld.maxxCap--;
            thisWorld.removeObject(this);//removes itself from world
        } else {//will count down the timer, and check for if a Star is touching the Explosion
            timer--;//reduces the timer until it reaches 0
            checkCollision();//will then check to see if a StarBang kills the explosion
        }//endif timer
    }//end of method countdown
    
    /**
     * Checks to see if a Star is touching the Explosion, and removes itself from gameplay if it's touching
     */
    private void checkCollision()
    {//local variables
        Actor star = getOneIntersectingObject(Star.class); //fair code, to not drain all of the Stars lives
        if ( star != null) {//if result is NOT void
            Gameplay thisWorld = (Gameplay) getWorld();//calls upon the Gameplay world for alterations
            //reduces the current bomb and max population, to reflect the disappearing explosion
            thisWorld.bombCap--;
            thisWorld.maxxCap--;
            thisWorld.removeObject(this);//removes itself from world
        }//endif collision removal
    }//end of method checkCollision
}//end of actor Explosion