import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Launches a pellet fired by a Pigeon, that poses a hazard to the Star actor
 * 
 * @Andrew I Letailleur
 * @15/03/2015, Version 2-03
 */
public class Pellet extends Projectile
{
    //class level attributes
    public int direction, speed;
    private int fuse = 3;//innitial fuse value
    
    /**
     * Set ups the Pellet's innitial appearance
     */
    public Pellet() // Do NOT do Void
    {//local variables
        //Original Image Size is set at 72 x 42
        GreenfootImage image = getImage();
        image.scale(image.getWidth()-46, image.getHeight()-31);
        setImage(image);        
        
        //direction = dir;
        //setRotation(direction);
        speed = 1;
    }//end of method Pellet
    
    /**
     * Act - do whatever the Pellet wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {//local variables
        move(speed);
        //setLocation(getX(), getY() - speed); old code
        checkCollision();
    }//end of method act
    
    /**
    * will play a sound file and create a delay to allow the file to play
    */
    private void play(String sound, int delayTime)
    {//will play a sound file and create a delay
        //to allow the file to play
        Greenfoot.playSound(sound);
        Greenfoot.delay(delayTime);
    }//end of method Play
    
    /**
     * Checks to see if Pellet is colliding with another Actor or at Edge of the World
     */
    public void checkCollision()
    {//local variables
        //returns actors that are touching the Pellet
        Actor star = getOneIntersectingObject(Star.class);
        Actor stalk = getOneIntersectingObject(Stalker.class);
        Actor bomb = getOneIntersectingObject(Bomb.class);
        Actor wolf = getOneIntersectingObject(Wolf.class);
        Actor bat = getOneIntersectingObject(Bat.class);
        if(atWorldEdge(this))//if Bullet is at the Edge of the World from the Projectile Super Class
            removePellet();//removes Bullet from Gameplay
        else if (fuse < 3)//If fuse is lower than standard
            checkFuse();
        else if (fuse > 2){//if fuse is at the standard value
            if( (star != null) || (stalk != null) || (bomb != null) || (wolf != null)|| (bat != null) )
                fuse = 2;//lowers the fuse by a notch
       }//end pellet check conditions
    }//end of method checkCollision
    
    /**
     * Checks the current fuse level. If it's at 0, it removes the bullet. If above 0, will lower the fuse
     */
    public void checkFuse()
    {//local variables
        if (fuse <= 0)//above 0
            removePellet();
        else //lowers fuse by 1
            fuse--;
        //endif fuse check
    }//end of method checkFuse
    
    /**
     * Removes the Pellet object from the world
     */
    public void removePellet()
    {//local variables
        play("Splat.wav", 0);//plays the splat sound
        Gameplay thisWorld = (Gameplay) getWorld();//calls upon the Gameplay world for alterations
        thisWorld.removeObject(this);//removes itself from world
    }//end of method removePellet
}//end of actor Pellet