import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * An enemy actor that will constantly home in on the star until it is shot or touched by the star's 'light'
 * 
 * @Andrew I Letailleur
 * @13/03/2015, Version 2-04
 */
public class Stalker extends Actor
{
    //class level attributes
    private int selfX, selfY, dying, bite;
    
    /**
     * Set ups the Stalker's innitial appearance
     */
    public Stalker()
    {//local variables
        //Original Image Size is set at 165 x 324
        GreenfootImage image = getImage();
        image.scale(image.getWidth() / 4, image.getHeight() / 4);
        setImage(image);
        
        dying = 2;
        bite = 1;
    }//end of method Stalker
    
    /**
    * will play a sound file and create a delay
    * to allow the file to play
    */
    private void play(String sound, int delayTime)
    {//will play a sound file and create a delay
       //to allow the file to play
       Greenfoot.playSound(sound);
       Greenfoot.delay(delayTime);
    }//end of method Play
    
    /**
     * Act - do whatever the Stalker wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {//local variables
        storeCurrentPosition();//sets current position for future reference
        //stalkStar();
        if ( (dying <= 0) && (bite !=1) )//call upon death by wolves condition
            wolfStalker();
        else if (dying <= 0)
            killStalker();//kills the Stalker via natural causes
        else if (dying < 2)
            dying--;
        else
            stalkStar();//stalks star once all checks are passed
        //endif stalk statement
    }//end of method act

    private void storeCurrentPosition()
    {//local variables
        //stores current position in order to aid in stalking the star
        selfX = getX();
        selfY = getY();
    }//end of method storeCurrentPosition
    
    /**
     * Returns a value that aids in tracking the star
     */
    private int trackSelfX()
    {//local variables
        return selfX;
    }//end of method trackSelfX
    private int trackSelfY()
    {//local variables
        return selfY;
    }//end of method trackSelfY
    
    /**
     * Moves the Stalker towards the Star's current location
     */
    private void stalkStar()
    {//local variables
        Gameplay thisWorld = (Gameplay) getWorld();
        if (thisWorld.player.trackStarX() - trackSelfX() > 0) {
             setLocation(getX() +1, getY());
        } else if (thisWorld.player.trackStarX() - trackSelfX() < 0) {
            setLocation(getX() -1, getY());
        }//endif stalk X
        if (thisWorld.player.trackStarY() - trackSelfY() > 0) {
            setLocation(getX(), getY() +1);
        } else if (thisWorld.player.trackStarY() - trackSelfY() < 0) {
            setLocation(getX(), getY() -1);
        }//endif stalk Y
        checkCollision();
    }//end of method stalkStar
    
    /**
     * Checks to see if Stalker is touching a hazard, and will remove itself from Gameplay if touched
     */
    private void checkCollision()
    {//local variables
        //returns actors that are touching the Stalker
        Actor shot = getOneIntersectingObject(Bullet.class);
        Actor bang = getOneIntersectingObject(Explosion.class);
        Actor wolf = getOneIntersectingObject(Wolf.class);
        Actor star = getOneIntersectingObject(StarBang.class);
        if (shot != null) //if a bullet hits, call upon death by bullet method
            bulletStalker();
        else if (wolf !=null) {//if a wolf bites, trigger the condition for death by wolves
            dying--;
            bite = 0;//sets bite to 0, which will trigger a wolf death condition
        } else if ( (bang !=null) || (star !=null) )//trigger the condition for death by hazards
            dying--;
        else if ( (dying <= 0) && (bite !=1) )//call upon death by wolves condition
            wolfStalker();
        else if (dying <= 0)//call upon death by hazards condition
            killStalker();
        //endif kill conditions
    }//end of method checkCollision
    
    /**
     * Removes Stalker from play if the actor touches a bullet object
     */
    public void bulletStalker()
    {//local variables
        play("stalkerDies.wav", 0);//plays the Stalkers death cry
        Gameplay thisWorld = (Gameplay) getWorld();//calls upon the Gameplay world for alterations
        Flash flash = new Flash();//adds a flash for any bullet that hits the Stalker
        thisWorld.addObject(flash, getX(), getY());//places the flash where the actor is currently at
        
        //reduces the current stalker and max population, to reflect the reduced amount of enemiess around
        thisWorld.stalkerCap--;
        thisWorld.maxxCap--;
        thisWorld.removeObject(this);//removes itself from world
        
        thisWorld.killCount++;//adds a kill to the kill counter
        thisWorld.Stalker(); //add to score method
    }//end of method bulletStalker
    
    /**
     * Removes Stalker from play if another hazard triggers it's death condition
     */
    public void killStalker()
    {//local variables
        play("stalkerDies.wav", 0);//plays the Stalkers death cry
        Gameplay thisWorld = (Gameplay) getWorld();//calls upon the Gameplay world for alterations
        //reduces the current stalker and max population, to reflect the reduced amount of enemiess around
        thisWorld.removeObject(this);
        thisWorld.stalkerCap--;
        thisWorld.maxxCap--;
    }//end of method killStalker
    
    /**
     * Removes Stalker from play if a wolf triggers a special death condition
     */
    public void wolfStalker()
    {//local variables
        play("wolfFeed.wav", 0);//plays a sound where a wolf chomps on the stalker
        Gameplay thisWorld = (Gameplay) getWorld();
        //reduces the current stalker and max population, to reflect the reduced amount of enemiess around
        thisWorld.removeObject(this);
        thisWorld.stalkerCap--;
        thisWorld.maxxCap--;
    }//end of method wolfStalker
}//end of actor Stalker
