import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * The main 'Star' of the game. Fires bullets, spawns a StarBang if touched with lives remaining, and fragile.
 * Oh, and it's supposedly 'drunk', given the nature of the game. Might add a spin feature on launching.
 * 
 * @Andrew I Letailleur
 * @13/03/2015, Version 2-03
 */
public class Star extends Actor
{
   //class level attributes
   
   //Image stuff
   private GreenfootImage image1;
   private GreenfootImage image2;
   private GreenfootImage image3;
   
   //important game stuff
   private byte lives, level;
   public int starX, starY = 300;
   
   private byte GunX, GunY = 0;
   
   public int kills = 0;
   
   //refresh mechanics
   private byte reload, regen = 0;
   
   private double check;
   /**
    * Constructor for objects of class Star
    * sets up initial values for the attributes, and other stuff
    */
   public Star()
   {//local variables
      //set up all instance variables when the object is created
      image1 = new GreenfootImage("Star1.png");
      image2 = new GreenfootImage("Star2.png");
      image3 = new GreenfootImage("Star3.png");
      setImage(image1); // all new image widths should be at 66 x 68
      
      //GreenfootImage image = getImage();
      //image.scale(image.getWidth()-200, image.getHeight()-200); old code that may crash game
      //setImage(image);
      
      lives=1;
      level=0;
      check=0;
      //reload=0;
      //regen=0;
   }//end of constructor Star
   
   /**
     * Act - do whatever the Star wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
   public void act() 
   {//local variables
       // Add your action code here.
       storeCurrentPosition();
       checkMove();
       if(reload>=1)
         reload--;
       if(regen>=1)
         regen--;
   }//end of method act
   
   /**
    * will play a sound file and create a delay
    * to allow the file to play
    */
   private void play(String sound, int delayTime)
   {//will play a sound file and create a delay
       //to allow the file to play
       Greenfoot.playSound(sound);
       Greenfoot.delay(delayTime);
   }//end of method Play
   
   /**
    * Changes the star's image if it gains or loses a life
    */
   private void updateImage()
   {//local variables
      if(lives<=1)
         setImage(image1);
      else if(lives==2)
         setImage(image2);
      else if(lives>=3)
         setImage(image3);
      //endif image conditions
   }//end of method updateImage

   /*Start of section statistics
    */
   /**
    * Special attributes from the Star, returning lives and level total for the Gameplay World
    */
   public int getLives()
   {//returns lives for the 'game over/golden ending' condition
       return lives;
   }//end of method getLives
   private void updateLives(int change)
   {//updates the current amount of lives and changes the stars appearence as a visual life guide
       lives += change;
       updateImage();
   }//end of method updateLives
   public int getLevel()
   {//returns the star level for the Gameplay
       return level;
   }//end of method getLevel
   public int getKills()
   {//returns the kill variable so the world can record the total
       return kills;
   }//end of method getLevel
   public void updateKills(int change)
   {//updates the current amount of kills so that the level up condition is checked
       kills += change;
   }//end of method updateKills(int change)
   /*End of section statistics
    */
   
   /*Start of section tracker
    */
   /**
    * Stores the current position of the Star, for reference by the Stalker class
    */
   private void storeCurrentPosition()
   {
       starX = getX();
       starY = getY();
   }//end of method storeCurrentPosition
   public int trackStarX()
   {//returns the horizontal value
       return starX;
   }
   public int trackStarY()
   {//returns the vertical value
       return starY;
   }
   /*End of section tracker
    */
   
   /**
    * Checks the keyboard imput from the player, and moves the star accordingly to the imput recieved
    */
   private void checkMove()
   {//local variables
       Gameplay thisWorld = (Gameplay) getWorld();//calls upon the Gameplay world for alterations
       
       if( (Greenfoot.isKeyDown("left")) && (Greenfoot.isKeyDown("right")) ){
           GunX = 0;
       }//endif stopgap
       else if(Greenfoot.isKeyDown("left")) {//if the left arrow key is pressed
           setLocation(getX() -2, getY());
           starX = starX -2;
           thisWorld.updatePlayerX();
           
           GunX = -1;
       }//endif left move
       else if(Greenfoot.isKeyDown("right")) {//if the right arrow key is pressed
           setLocation(getX() +2, getY());
           starX = starX +2;
           thisWorld.updatePlayerX();
           
           GunX = 1;
       }//endif right move
       else {
           GunX = 0;
       }//endif vertical check
       if( (Greenfoot.isKeyDown("up")) && (Greenfoot.isKeyDown("down")) ){
           GunY = 0;
       }//endif stopgap
       else if(Greenfoot.isKeyDown("up")) {//if the up arrow key is pressed
           setLocation(getX() , getY() -2);
           starY = starY -2;
           thisWorld.updatePlayerY();
           
           GunY = -1;
       }//endif up move
       else if(Greenfoot.isKeyDown("down")) {//if the down arrow key is pressed
           setLocation(getX() , getY() +2);
           starY = starY +2;
           thisWorld.updatePlayerY();
           
           GunY = 1;
       }//endif down move
       else{
           GunY = 0;
       }//endif horizontal check
       if(Greenfoot.isKeyDown("space"))//if the spacebar is pressed
           fireGun();
       //endif check gun trigger

       /* if( (Greenfoot.isKeyDown("x")) && (lives < 3))
        * updateLives(+1);
        * endif unlimited lives cheat
        */
       checkCollision();
   }//end of method checkMove
      
    /**
     * Checks to see if a hazard is touching the Star, and to take away a life if a hazard touches the Star
     * Also includes a brief recovery period, to prevent an 'all lives lost' glitch
     */
   private void checkCollision()
   {//local variables
       //returns actors that are touching the Star
       Actor stalk = getOneIntersectingObject(Stalker.class);
       Actor wolf = getOneIntersectingObject(Wolf.class);
       Actor bat = getOneIntersectingObject(Bat.class);
       Actor boom = getOneIntersectingObject(Explosion.class);
       Actor shit = getOneIntersectingObject(Pellet.class);
       
       Gameplay thisWorld = (Gameplay) getWorld();//calls upon the Gameplay world for alterations
       if (regen == 0) {//checks to see if the star isn't hurt, to prevent instant gameover on collisions
           if ( (stalk != null) || (wolf != null) || (bat != null) || (boom != null) || (shit != null) ) {
              updateLives(-1);//takes away a life
              regen = 60;//sets the regen so that the star doesn't die instantly
              //endif touching enemy actors
              if(lives>0) {
               play("lifeLost.wav", 0);//plays a metal groin to signify a star shining in pain
               shine();//spawns a StarBang if star is hit by an enemy
               thisWorld.Penalty();//reduces 'score' variable by a margain of *0.9
               //endif life deduction
              } else {//will play if gameover condition is met
               play("starDies.wav", 0);
               play("Dead.wav", 0);
              }//endif gameover condition
           }//endif hit collision
       }//endif regen condition
       
       checkKills();
   }//end of method checkCollision
   
   /**
    * Spawns a StarBang where the Star is positioned, to kill of hazards while the Star recovers
    */
   private void shine() //hope this fixes possible glitches
   {
        Gameplay thisWorld = (Gameplay) getWorld();//calls upon the Gameplay world for alterations
        
        StarBang bang = new StarBang();
        thisWorld.addObject(bang, starX, starY);
   }//end of method shine
   
   /**
    * Constantly checks for Level Up conditions via the kill counter
    */
   public void checkKills()
   {
       Gameplay thisWorld = (Gameplay) getWorld();//calls upon the Gameplay world for alterations
       if(kills != thisWorld.killCounter())
            kills = thisWorld.killCounter();
       //endif kill counter
       if(level == 0)//checks for move to Green Forest condition
            checkBlue();
       else if(level == 1)//checks for move to Red Cave condition
            checkGren();
       else if(level == 2)//checks for Ending condition
            checkRed();
       //endif level condition
   }//end of method checkWorld
   
   /**
    * checkBlue, checkGren & checkRed all change the gameplay world if conditions are met
    * checkBlue checks for Green Forest condition
    */
   public void checkBlue()
   {
       if (kills>=20) {//20 kills required
           play("Yay.wav", 5);
           level++;
           updateLives(+1);
           Gameplay thisWorld = (Gameplay) getWorld();//calls upon the Gameplay world for alterations
           Greenfoot.setWorld(new gTree(this));//changes the gameplay world to 'Green Forest'
           //sets the max cap back to 0
           thisWorld.stalkerCap = 0;
           thisWorld.bombCap = 0;
       }//endif kill counter
   }//end of method checkBlue
   /**
    * checkGren checks for Red Cave condition
    */
   public void checkGren()
   {
       if (kills>=35) {//20 + 35 = 55 kills required
           play("Yay.wav", 5);
           level++;
           updateLives(+1);
           Gameplay thisWorld = (Gameplay) getWorld();//calls upon the Gameplay world for alterations
           Greenfoot.setWorld(new rCave(this));//changes the gameplay world to 'Green Forest'
           //sets the max cap back to 0
           thisWorld.stalkerCap = 0;
           thisWorld.bombCap = 0;
           thisWorld.pigeonCap = 0;
           thisWorld.wolfCap = 0;
       }//endif kill counter 
   }//end of method checkGren
   /**
    * checkRed checks for Ending condition, and will play a victory speech once conditions are met
    */
   public void checkRed()
   {
       if (kills>=45) {//55 + 45 = 100 kills required
           level++;
           //Gameplay thisWorld = (Gameplay) getWorld();
           //check = thisWorld.scoreCounter();
           if(getLives() >= 3)//-50 for 'safety gap' in scoring, just in case
               play("Twinner.wav", 5);
           else {
               play("Ywinner.wav", 5);
               //Greenfoot.setWorld(new rCave(this)); ending screen check condition?
               //Greenfoot.setWorld(new YouWinner(this));
               //Gameplay thisWorld = (Gameplay) getWorld();
          
               //thisWorld.stalkerCap = 0;
               //thisWorld.bombCap = 0;
               //thisWorld.pigeonCap = 0;
               //thisWorld.wolfCap = 0;
               //thisWorld.batCap = 0;
               //thisWorld.lavaCap = 0;
           }//endif victory speech 
       }//endif kill counter 
   }//end of method checkRed
   /*End of section level check
    */
   /**
    * fires a bullet if the Spacebar is pressed by the star.
    */
   public void fireGun()
   {
       //local variables
       if (reload==0) {
           play("Gunshot.wav", 5);//plays a gunshot sound
           if ( (GunX == 0) && (GunY == -1) ) {//if the up arrow key is pressed
               Bullet bullet = new Bullet();
               getWorld().addObject (bullet, getX(), getY() -40);
               bullet.setRotation(270);//North Rotation
               reload = 50;
           } else if ( (GunX == 1) && (GunY == -1) ) {//if the up/right arrow key is pressed
               Bullet bullet = new Bullet();
               getWorld().addObject (bullet, getX() +3, getY() -4);
               bullet.setRotation(315);//North-West Rotation
               reload = 50;
           } else if ( (GunX == 1) && (GunY == 0) ) {//if the right arrow key is pressed
               Bullet bullet = new Bullet();
               getWorld().addObject (bullet, getX() +30, getY());
               bullet.setRotation(0);//West Rotation
               reload = 50;
           } else if ( (GunX == 1) && (GunY == 1) ) {//if the right/down arrow key is pressed
               Bullet bullet = new Bullet ();
               getWorld().addObject (bullet, getX() +3, getY() +7);
               bullet.setRotation(45);//West-South Rotation
               reload = 50;
           } else if ( (GunX == 0) && (GunY == 1) ){//if the down arrow key is pressed
               Bullet bullet = new Bullet();
               getWorld().addObject (bullet, getX(), getY() +27);
               bullet.setRotation(90);//South Rotation
               reload = 50;
           } else if ( (GunX == -1) && (GunY == 1) ) {//if the down/left arrow key is pressed
               Bullet bullet = new Bullet ();
               getWorld().addObject (bullet, getX() -3, getY() +7);
               bullet.setRotation(135);//South-East Rotation
               reload = 50;
           } else if ( (GunX == -1) && (GunY == 0) ) {//if the left arrow key is pressed
               Bullet bullet = new Bullet ();
               getWorld().addObject (bullet, getX() -30, getY());
               bullet.setRotation(180);//East Rotation
               reload = 50;
           } else if ( (GunX == -1) && (GunY == -1) ) {//if the left/up arrow key is pressed
               Bullet bullet = new Bullet ();
               getWorld().addObject (bullet, getX() -0, getY() -4);
               bullet.setRotation(225);//East-North Rotation
               reload = 50;
           } else {//spawns a bullet in a random direction if no direction imput is chosen, a gamble shot
               Bullet bullet = new Bullet ();
               getWorld().addObject (bullet, getX(), getY());
               bullet.turn(Greenfoot.getRandomNumber(360));//Drunken Rotation
               reload = 50;
           }//endif bullet spawn
       }//endif reload check
       //consider adding a 'set aim to X' code on checkMove as well
   }//end of method fireGun
}//end of actor Star (the Drunken)